@@ -419,7 +419,8 @@ impl DirectoryBuffer {
 
 #[derive(Debug, Clone, Eq, PartialEq, Serialize, Deserialize)]
 pub enum InternalMsg {
-    AddDirectory(String, DirectoryBuffer),
+    AddLastFocus(String, Option<String>),
+    SetDirectory(DirectoryBuffer),
     HandleKey(Key),
 }
 
@@ -1517,7 +1518,8 @@ pub struct App {
     pub version: String,
     pub config: Config,
     pub pwd: String,
-    pub directory_buffers: HashMap<String, DirectoryBuffer>,
+    pub directory_buffer: Option<DirectoryBuffer>,
+    pub last_focus: HashMap<String, Option<String>>,
     pub selection: IndexSet<Node>,
     pub msg_out: VecDeque<MsgOut>,
     pub mode: Mode,
@@ -1534,8 +1536,13 @@ pub struct App {
 }
 
 impl App {
-    pub fn create(pwd: PathBuf, lua: &mlua::Lua, config_file: Option<PathBuf>) -> Result<Self> {
-        let config = lua::init(lua)?;
+    pub fn create(
+        pwd: PathBuf,
+        lua: &mlua::Lua,
+        config_file: Option<PathBuf>,
+        extra_config_files: Vec<PathBuf>,
+    ) -> Result<Self> {
+        let mut config = lua::init(lua)?;
 
         let config_file = if let Some(path) = config_file {
             path
@@ -1545,14 +1552,19 @@ impl App {
             PathBuf::from("/").join("etc").join("xplr").join("init.lua")
         };
 
-        let (config, load_err) = if config_file.exists() {
+        let config_files = std::iter::once(config_file).chain(extra_config_files.into_iter());
+
+        let mut load_errs = vec![];
+        for config_file in config_files {
             match lua::extend(lua, &config_file.to_string_lossy().to_string()) {
-                Ok(c) => (c, None),
-                Err(e) => (config, Some(e.to_string())),
+                Ok(c) => {
+                    config = c;
+                }
+                Err(e) => {
+                    load_errs.push(e.to_string());
+                }
             }
-        } else {
-            (config, None)
-        };
+        }
 
         let mode = match config.modes().get(
             &config
@@ -1582,7 +1594,7 @@ impl App {
 
         let pid = std::process::id();
         let session_path = dirs::runtime_dir()
-            .unwrap_or_else(|| "/tmp".into())
+            .unwrap_or_else(env::temp_dir)
             .join("xplr")
             .join("session")
             .join(&pid.to_string())
@@ -1604,11 +1616,12 @@ impl App {
         env::set_current_dir(&pwd)?;
         let pwd = pwd.to_string_lossy().to_string();
 
-        let app = Self {
+        let mut app = Self {
             version: VERSION.to_string(),
             config,
             pwd,
-            directory_buffers: Default::default(),
+            directory_buffer: Default::default(),
+            last_focus: Default::default(),
             selection: Default::default(),
             msg_out: Default::default(),
             mode,
@@ -1626,11 +1639,11 @@ impl App {
 
         fs::create_dir_all(app.session_path())?;
 
-        if let Some(err) = load_err {
-            app.log_error(err)
-        } else {
-            Ok(app)
+        for err in load_errs {
+            app = app.log_error(err)?
         }
+
+        Ok(app)
     }
 
     pub fn focused_node(&self) -> Option<&Node> {
@@ -1658,7 +1671,10 @@ impl App {
 
     fn handle_internal(self, msg: InternalMsg) -> Result<Self> {
         match msg {
-            InternalMsg::AddDirectory(parent, dir) => self.add_directory(parent, dir),
+            InternalMsg::SetDirectory(dir) => self.set_directory(dir),
+            InternalMsg::AddLastFocus(parent, focus_path) => {
+                self.add_last_focus(parent, focus_path)
+            }
             InternalMsg::HandleKey(key) => self.handle_key(key),
         }
     }
@@ -1812,14 +1828,17 @@ impl App {
         Ok(self)
     }
 
-    pub fn explore_pwd(self) -> Result<Self> {
+    pub fn explore_pwd(mut self) -> Result<Self> {
+        let focus = self.last_focus.get(self.pwd()).cloned().unwrap_or(None);
+        let pwd = self.pwd().clone();
+        self = self.add_last_focus(pwd, focus.clone())?;
         let dir = explorer::explore_sync(
             self.explorer_config().clone(),
             self.pwd().into(),
-            self.focused_node().map(|n| n.absolute_path().into()),
+            focus.map(PathBuf::from),
             self.directory_buffer().map(|d| d.focus()).unwrap_or(0),
         )?;
-        self.add_directory(dir.parent().clone(), dir)
+        self.set_directory(dir)
     }
 
     fn explore_pwd_async(mut self) -> Result<Self> {
@@ -1965,6 +1984,9 @@ impl App {
 
         match env::set_current_dir(&dir) {
             Ok(()) => {
+                let pwd = self.pwd().clone();
+                let focus = self.focused_node().map(|n| n.relative_path().clone());
+                self = self.add_last_focus(pwd, focus)?;
                 self.pwd = dir.to_string_lossy().to_string();
                 if save_history {
                     self.history = self.history.push(format!("{}/", self.pwd));
@@ -2281,8 +2303,19 @@ impl App {
         })
     }
 
-    pub fn add_directory(mut self, parent: String, dir: DirectoryBuffer) -> Result<Self> {
-        self.directory_buffers.insert(parent, dir);
+    pub fn set_directory(mut self, dir: DirectoryBuffer) -> Result<Self> {
+        self = self.add_last_focus(
+            dir.parent.clone(),
+            dir.focused_node().map(|n| n.relative_path().clone()),
+        )?;
+        if dir.parent == self.pwd {
+            self.directory_buffer = Some(dir);
+        }
+        Ok(self)
+    }
+
+    pub fn add_last_focus(mut self, parent: String, focused_path: Option<String>) -> Result<Self> {
+        self.last_focus.insert(parent, focused_path);
         Ok(self)
     }
 
@@ -2593,7 +2626,7 @@ impl App {
     }
 
     fn directory_buffer_mut(&mut self) -> Option<&mut DirectoryBuffer> {
-        self.directory_buffers.get_mut(&self.pwd)
+        self.directory_buffer.as_mut()
     }
 
     /// Get a reference to the app's pwd.
@@ -2603,7 +2636,7 @@ impl App {
 
     /// Get a reference to the app's current directory buffer.
     pub fn directory_buffer(&self) -> Option<&DirectoryBuffer> {
-        self.directory_buffers.get(&self.pwd)
+        self.directory_buffer.as_ref()
     }
 
     /// Get a reference to the app's config.
@@ -2629,11 +2662,6 @@ impl App {
         format!("{}\n", &self.mode.name())
     }
 
-    /// Get a reference to the app's directory buffers.
-    pub fn directory_buffers(&self) -> &HashMap<String, DirectoryBuffer> {
-        &self.directory_buffers
-    }
-
     /// Get a reference to the app's input buffer.
     pub fn input_buffer(&self) -> Option<String> {
         self.input_buffer.clone()
@@ -2784,70 +2812,27 @@ impl App {
     }
 
     pub fn write_pipes(&self) -> Result<()> {
-        // TODO optimize and test
-
         fs::create_dir_all(self.pipe().path())?;
-
         fs::write(&self.pipe().msg_in, "")?;
 
         let selection_str = self.selection_str();
-        // if last_app
-        //     .map(|a| a.selection_str() != selection_str)
-        //     .unwrap_or(true)
-        // {
         fs::write(&self.pipe().selection_out, selection_str)?;
-        // };
 
         let history_str = self.history_str();
-        // if last_app
-        //     .map(|a| a.history_str() != history_str)
-        //     .unwrap_or(true)
-        // {
         fs::write(&self.pipe().history_out, history_str)?;
-        // };
 
         let directory_nodes_str = self.directory_nodes_str();
-        // if last_app
-        //     .map(|a| a.directory_nodes_str() != directory_nodes_str)
-        //     .unwrap_or(true)
-        // {
         fs::write(&self.pipe().directory_nodes_out, directory_nodes_str)?;
-        // };
 
-        // if last_app
-        //     .map(|a| a.logs().len() != self.logs().len())
-        //     .unwrap_or(true)
-        // {
-        // let new_logs = self
-        //     .logs()
-        //     .iter()
-        //     .skip(last_app.map(|a| a.logs().len()).unwrap_or(0))
-        //     .map(|l| format!("{}\n", l))
-        //     .collect::<Vec<String>>()
-        //     .join("");
-
-        // let mut file = fs::OpenOptions::new()
-        //     .append(true)
-        //     .open(&self.pipe().logs_out)?;
-        // file.write_all(new_logs.as_bytes())?;
-        // };
         let logs_str = self.logs_str();
         fs::write(&self.pipe().logs_out, logs_str)?;
 
         let result_str = self.result_str();
-        // if last_app
-        //     .map(|a| a.result_str() != result_str)
-        //     .unwrap_or(true)
-        // {
         fs::write(&self.pipe().result_out, result_str)?;
-        // };
+
         let global_help_menu_str = self.global_help_menu_str();
-        // if last_app
-        //     .map(|a| a.global_help_menu_str() != global_help_menu_str)
-        //     .unwrap_or(true)
-        // {
         fs::write(&self.pipe().global_help_menu_out, global_help_menu_str)?;
-        // };
+
         Ok(())
     }
 
